body{margin:0;font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}.auth-main{height:100vh;background-color:#eaeaea}.auth-content{left:50%;padding:10vmin;position:fixed;text-align:center;top:50%;-webkit-transform:translate(-50%,-50%);transform:translate(-50%,-50%)}.auth-card{background-color:#fff;box-shadow:3px 3px 8px #888;border-radius:4px;width:400px;min-width:300px;text-align:center;padding:20px}.auth-logo{width:200px;margin-bottom:-10px}.auth-form{margin-top:35px}.auth-input-field{font-size:18px;margin-bottom:25px}body,html{height:100%;margin:0}.base-page{display:flex;flex-direction:column;background-color:#eaeaea;padding:20px;flex-grow:1;min-height:calc(100vh - 40px)}.base-container{background-color:#eaeaea;padding:10px}.network-header{padding-top:5px!important;padding-bottom:0!important}.network-detail{padding-top:0!important;padding-bottom:5px!important}
/*# sourceMappingURL=main.ad78b6b4.chunk.css.map */